import unittest
from mock import patch
from mock import MagicMock
import tests.util as util
import updateMergeProposal


class TestUpdateMergeProposal(unittest.TestCase):

    merge_proposal = \
        'https://code.launchpad.net/~mrazik/faux-dbus-test-runner/' + \
        'yar8/+merge/106113'

    def setUp(self):
        self.launchpadTrigger = MagicMock()
        self.launchpadTrigger.get_merge_proposal = MagicMock(
                    return_value='Launchpad_mp')
        self.launchpadTrigger.hide_jenkins_url = MagicMock(
                    return_value='http://s-jenkins/job/' +
                                 'faux-dbus-test-runner-ci/configure')
        self.launchpadTrigger.get_executed_test_runs = MagicMock(
                    return_value='')
        self.launchpadTrigger.format_message_for_mp_update = lambda x, y: y
        self.launchpad_trigger_patch = patch(
                    'updateMergeProposal.LaunchpadTrigger',
                    new=MagicMock(return_value=self.launchpadTrigger))
        self.LaunchpadTrigger = self.launchpad_trigger_patch.start()

    def tearDown(self):
        self.launchpad_trigger_patch.stop()

    def test_gibberish(self):
        """Gibberish on the command line produces usage message."""

        sys_argv = ['--oheck']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                updateMergeProposal.main()
            except SystemExit:
                pass
            self.assertTrue("usage" in stderr.getvalue())

    def test_no_commit_message(self):
        """Update merge proposal to show that no commit message was
        specified."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--commit-message-empty',
                    '--revision=66',
                    '--vote=Disapprove']
        with patch('sys.argv', sys_argv):
            updateMergeProposal.main()
            self.launchpadTrigger.change_mp_status.assert_called_with(
                'Launchpad_mp',
                "Needs review",
                "FAILED: Autolanding.\n" +
                "No commit message was specified.\n",
                "Re: [Merge] autolanding failed",
                '66',
                'Disapprove')

    def test_merge_failed(self):
        """Update merge proposal to show that merge failed."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--merge-failed',
                    '--revision=66',
                    '--vote=Disapprove']
        with patch('sys.argv', sys_argv):
            updateMergeProposal.main()
            self.launchpadTrigger.change_mp_status.assert_called_with(
               'Launchpad_mp',
               "Needs review",
               "FAILED: Autolanding.\n" +
               "Merging failed. More details in the following jenkins job:\n",
               "Re: [Merge] autolanding failed (merge failed)",
               '66',
               'Disapprove')

    def test_push_failed(self):
        """Update merge proposal to show that push failed."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--push-failed',
                    '--revision=66',
                    '--vote=Disapprove']
        with patch('sys.argv', sys_argv):
            updateMergeProposal.main()
            self.launchpadTrigger.change_mp_status.assert_called_with(
               'Launchpad_mp',
               "Needs review",
               "FAILED: Autolanding.\n" +
               "Pushing failed. More details in the following jenkins job:\n",
               "Re: [Merge] autolanding failed (push failed)",
               '66',
               'Disapprove')

    def test_landing_failed(self):
        """Update merge proposal to show that landing failed."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--landing-failed',
                    '--revision=66',
                    '--vote=Disapprove']
        with patch('sys.argv', sys_argv):
            updateMergeProposal.main()
            self.launchpadTrigger.change_mp_status.assert_called_with(
                        'Launchpad_mp',
                        "Needs review",
                        "FAILED: Autolanding.\n" +
                        "More details in the following jenkins job:\n",
                        "Re: [Merge] autolanding failed",
                        '66',
                        'Disapprove')

    def test_merged(self):
        """Update merge proposal to show that branch merged."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                                'faux-dbus-test-runner-ci/configure"',
                    '--merged',
                    '--revision=66',
                    '--vote=Approve']
        with patch('sys.argv', sys_argv):
            updateMergeProposal.main()
            self.launchpadTrigger.change_mp_status.assert_called_with(
                                      'Launchpad_mp',
                                      "Merged",
                                      "PASSED: Autolanding.\n" +
                                      "Branch merged.",
                                      "Re: [Merge] branch merged",
                                      '66',
                                      'Approve')

    def test_unknown_merge_proposal(self):
        """The given merge proposal wasn't found; error out with message."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~oheck/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--merged',
                    '--revision=66',
                    '--vote=Disapprove']
        error_message = 'Error: merge proposal relted to this branch ' + \
                        'was not found'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch.object(self.LaunchpadTrigger.return_value,
                             'get_merge_proposal') as get_merge_proposal:
            get_merge_proposal.return_value = None
            try:
                updateMergeProposal.main()
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())

    def test_no_merge_proposal_specified(self):
        """No merge proposal specified; error out with message."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--merged',
                    '--revision=66',
                    '--vote=Disapprove']
        error_message = 'error: argument -m/--merge-proposal is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            with self.assertRaises(SystemExit) as e:
                updateMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)

    def test_no_branch_and_none_found(self):
        """No branch specified and none can be inferred."""

        sys_argv = ['updateMergeProposal.py',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                      'faux-dbus-test-runner/yar8/+merge/' +
                                      '106113"',
                    '--job-url="http://s-jenkins/job/' +
                               'faux-dbus-test-runner-ci/configure"',
                    '--merged',
                    '--revision=66',
                    '--vote=Disapprove']
        error_message = ('Error: unable to get branch from merge proposal.\n' +
                            'Please specify manually using --branch.')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch.object(self.LaunchpadTrigger.return_value,
                             'get_branch_from_mp') as get_branch_from_mp:
            get_branch_from_mp.return_value = None
            with self.assertRaises(SystemExit) as e:
                updateMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)

    def test_no_revision_specified(self):
        """No revision specified; exit with message."""

        sys_argv = ['updateMergeProposal.py',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/' +
                                       'faux-dbus-test-runner/yar8/+merge/' +
                                       '106113"',
                    '--job-url="http://s-jenkins/job/' +
                                'faux-dbus-test-runner-ci/configure"',
                    '--vote=Disapprove']
        error_message = 'error: argument -v/--revision is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            with self.assertRaises(SystemExit) as e:
                updateMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)
